@@ -3,6 +3,36 @@ pub mod dp {
     //! This is a module for dynamic programming.
     
     use std::collections::VecDeque;
+
+    fn gcd_multi(v: Vec<u32>) -> u32 {
+        let mut result = v[0];
+        for i in 1..v.len() {
+            result = gcd(result, v[i]);
+        }
+        result
+    }
+
+    fn gcd(a: u32, b: u32) -> u32{
+        if b == 0 {
+            a
+        } else {
+            gcd(b, a % b)
+        }
+    }
+    
+    #[test]
+    
+    fn test_gcd() {
+        assert_eq!(gcd(20, 10), 10);
+        assert_eq!(gcd(55, 5), 5);
+        assert_eq!(gcd(991, 997), 1);
+    }
+
+    #[test]
+    fn test_gcd_multi() {
+        assert_eq!(gcd_multi(vec![5, 10, 20]), 5);
+        assert_eq!(gcd_multi(vec![131, 863, 887]), 1);
+    }
     /// Finds subsets sum of a target value. It can accept negative values.
     /// 
     /// # Example
@@ -83,31 +113,26 @@ pub mod dp {
         }
     }
 
-    fn filter_j_idx(n: usize, a:  &Vec<u32>) -> (Vec<usize>, u32){
+    pub fn filter_j_idx(n: usize, a:  &Vec<u32>) -> (Vec<usize>, u32){
         // a_min is the minimum number in an except for zero.
         let mut a_min = a.iter().max().unwrap();
-        let mut remainder: usize = 0;
+        let mut a_no_zero: Vec<u32> = vec![];
         for i in a{
             if i > &0 {
                 if a_min > &i {
                     a_min = &i
                 }
-            }
-
-            if i % 2 == 0 && remainder == 0{
-                remainder = 0;
-            } else {
-                remainder = 1;
+                a_no_zero.push(*i);
             }
         }
         let mut j_indexes: Vec<usize> = vec![];
-
+        let gcd = gcd_multi(a_no_zero);
         // j of the range of 1 to a_min-1 must be zero.
         // For example, if a_min = 10, there is no way to make sum 5.
         // Also, if j == 8 and target = 10 and a_min=5, we can't reach 10.
         // If all the numbers are even, j should be even.
         for j in 0..n+1{
-            if (j as u32 >= *a_min && j as u32 <=  n as u32 - *a_min && j % 2 <= remainder) || j as u32 == 0 || j == n  {
+            if (j as u32 >= *a_min && j as u32 <=  n as u32 - *a_min && j as u32 % gcd == 0) || j as u32 == 0 || j == n  {
                 j_indexes.push(j)
             }
         }
@@ -116,17 +141,25 @@ pub mod dp {
 
     #[test]
     fn test_filter_j_idx(){
-        let (result, a_min) = filter_j_idx(10, &vec![3, 4, 5, 6, 7 ,8, 9, 10]);
+        let (result, _a_min) = filter_j_idx(10, &vec![3, 4, 5, 6, 7 ,8, 9, 10]);
         let answer: Vec<usize> = vec![0, 3, 4, 5, 6, 7, 10];
         assert_eq!(result, answer);
 
-        let (result, a_min) = filter_j_idx(5, &vec![3, 4, 5]);
+        let (result, _a_min) = filter_j_idx(5, &vec![3, 4, 5]);
         let answer: Vec<usize> = vec![0, 5];
         assert_eq!(result, answer);
 
-        let (result, a_min) = filter_j_idx(10, &vec![0, 2, 4, 6, 8]);
+        let (result, _a_min) = filter_j_idx(10, &vec![0, 2, 4, 6, 8]);
         let answer: Vec<usize> = vec![0, 2, 4, 6, 8, 10];
         assert_eq!(result, answer);
+
+        let (result, _a_min) = filter_j_idx(20, &vec![10, 20, 30, 40, 50]);
+        let answer: Vec<usize> = vec![0, 10, 20];
+        assert_eq!(result, answer);
+
+        let (result, _a_min) = filter_j_idx(8, &vec![2, 3, 5, 7]);
+        let answer: Vec<usize> = vec![0, 2, 3, 4, 5, 6, 8];
+        assert_eq!(result, answer);
     }
     
     /// Finds subsets sum of a target value. It can't accept negative values but relatively faster.
